module.exports = function Engine () {
	
	var groupHelper = require("./group.js")();


	var savedGame = [];
	var savedCurrentColor = 0;
	var saveLastPlayedStone = null;
	/*
	 * public
	 */
	
	var lastPlayedStone = null;
	var AUTO_REMOVE_DEAD_STONES = true;
	this.asStone = function (move) {
			var m = move.B;
			var c = 1; // black
			if (!m) {
				m = move.W;
				c = 2; // white
			} if (! m) {
				return null;
			}
			if ((m === 'tt') || (m==='')){return null;}  // '' = "tt" = pass
			
			var x = m.charCodeAt(0) - 97;
			var y =  (m.charCodeAt(1) - 97) ;
			return {x:x,y:y,c:c};
		};
	this.update = function (game,stone) {

		if (stone) {
			lastPlayedStone = stone;
			var sx = game[0].length;
			var sy = game.length;
			var newGameState = this.newGameXY(sx,sy);
			// copy the game
			for  (var y = 0; y< sy; y++) {
				var l = [];
				for  (var x = 0; x< sx; x++) {
					newGameState[y][x] = game[y][x];
				}
			}
			// place current move
			newGameState[stone.y][stone.x] = stone.c;

			// check dead stones
			var st_groups = groupHelper.buildGroups(newGameState);
			
			if (AUTO_REMOVE_DEAD_STONES) {

				for (var i = 0 ; i< st_groups.length;i++) {
					var st_group = st_groups[i];
					if (st_group.liberties.length === 0) {
						if(st_group.stones[0].c !== stone.c) {
							removeGroup(newGameState,st_group.stones);
						}
					}
				}
				// build state again... (as we just removed some groups...)
				st_groups = groupHelper.buildGroups(newGameState);
			}


			return {game:newGameState, st_groups:st_groups};
		} else {
			console.log("Engine.update : ERROR Undefined stone !!! ");
			return {game:game, st_groups:groupHelper.buildGroups(game)};
		}
	};

	this.newGameXY = function(sx,sy) {
		var g = [];
		for  (var y = 0; y< sy; y++) {
			var l = [];
			for  (var x = 0; x< sx; x++) {
				l[x] = 0;
			}
			g[y] = l;
		}
		return g;
	};

	this.newGame = function(gameSize) {
		var g = [];
		for  (var y = 0; y< gameSize; y++) {
			var l = [];
			for  (var x = 0; x< gameSize; x++) {
				l[x] = 0;
			}
			g[y] = l;
		}
		return g;
	};

	this.pushGame = function(game,currentColor) {
		saveLastPlayedStone = lastPlayedStone;
		
		var sx = game[0].length;
		var sy = game.length;
		savedGame = this.newGameXY(sx,sy);

		

		savedCurrentColor = currentColor;
		for  (var y = 0; y< sy; y++) {
			for  (var x = 0; x< sx; x++) {
				savedGame[y][x] = game[y][x];
			}
		}
	};

	this.popGame = function() {

		var sx = savedGame[0].length;
		var sy = savedGame.length;
		var tmp = this.newGameXY(sx,sy);

		
		for  (var y = 0; y< sy; y++) {
			var l = [];
			for  (var x = 0; x< sx; x++) {
				tmp[y][x] = savedGame[y][x];
			}
		}
		
		return {game:tmp, c:savedCurrentColor, lastPlayedStone:saveLastPlayedStone};
	};

	this.getStoneColorAt = function(game, pos) {
		return game[pos.y][pos.x];
	};

	function removeGroup(game,group) {
		for (var j = 0;j<group.length;j++) {
			var stone = group[j];
			game[stone.y][stone.x] = 0; // clear stone
		}
	}
	
	function extractGameRect(game, x, y, width, height) {
		if (x < 0 ) return null;
		if (x+width > game[0].length ) return null;
		if (y < 0 ) return null;
		if (y+height > game.length ) return null;
		var res = [];
		for(var j = 0; j<height; j++) {
			var line = [];
			for(var i = 0;i< width ; i++) {
				line.push(game[j+y][i+x]);
			}
			res.push(line);
		}
		return res;
	}


	return {
		update : this.update,
		newGameXY : this.newGameXY,
		asStone : this.asStone,
		extractGameRect : extractGameRect
	};

	
};
