module.exports = function GroupHelper () {
	/* -----------------------------------------------------------
	 * public API
	 *
	 */
	
	this.buildGroups = function (game) {
		var groups = [];
		for (var y = 0; y < game.length; y++) {
			for(var x = 0; x<game[0].length; x++) {
				var c = game[y][x];
				if (c !== 0) {
					var stone = {x:x,y:y,c:c};
					addStoneToTheRightGroup(game,groups,stone);
				}
			}
		}
		
		// build a strucutre for each group, with stones
		// that belong to group and array of liberties of the group
		var st_groups = [];
		for (var i = 0;i< groups.length;i++) {
			var group = groups[i];
			var st_group = {
				stones : group,
				liberties : findGroupLiberties(game,group)
			};
			st_groups.push(st_group);
		}
		return st_groups;
	};

	this.groupsToString = function(groups) {
		var str ="";
		for(var j = 0;j<groups.length;j++) {
			var group = groups[j];
			str+="\n";
			for(var i = 0;i<group.length;i++) {
				var stone = group[i];
				str += "{"+stone.x+","+stone.y+"} ";
			}
		}
		return str;
	};

	function posToString (arrayOfpos) {
		var str ="";
		for(var i = 0;i<arrayOfpos.length;i++) {
			var pos = arrayOfpos[i];
			str += "{"+pos.x+","+pos.y+"} ";
		}
		return str;
	}

	this.stGroupsToString = function(st_groups) {
		var str ="";
		for(var j = 0;j<st_groups.length;j++) {
			var st_group = st_groups[j];
			str+="\n{\n";
			str+="  stones    : "+posToString(st_group.stones)+"\n";
			str+="  liberties : "+posToString(st_group.liberties)+"\n";
			str+="}\n";
		}
		return str;
	};

	function findGroupLiberties(game,group) {
		var liberties = []; // list of pos (pos={x,y});
		
		var hashtable = {
			hash: {},
			exists: function(key){
				return this.hash.hasOwnProperty(key);
			},
			get: function(key){
				return this.exists(key)?this.hash[key]:null;
			},
			put: function(key, value){
				this.hash[key] = value;
			}
		};

		for(var i = 0;i<group.length;i++) {
			var stone = group[i];
			var stoneLiberties = findStoneLiberties(game,stone);
			for (var j=0;j<stoneLiberties.length;j++) {
				var stoneLiberty = stoneLiberties[j];
				var stoneLibertyStr = stoneLiberty.x+"X"+stoneLiberty.y;
				if (! hashtable.exists(stoneLibertyStr)) {
					hashtable.put(stoneLibertyStr,"x");
					liberties.push(stoneLiberty);
				}
			}
		}
		return liberties;
	}

	function findStoneLiberties(game,stone) {
		var liberties = []; // list of pos (pos={x,y});
		
		if (uf(game,stone)) {
			liberties.push({x:stone.x, y:stone.y-1});
		}
		if (bf(game,stone)) {
			liberties.push({x:stone.x, y:stone.y+1});
		}
		if (rf(game,stone)) {
			liberties.push({x:stone.x+1, y:stone.y});
		}
		if (lf(game,stone)) {
			liberties.push({x:stone.x-1, y:stone.y});
		}
		
		return liberties;
	}

	function isGroupDead(game,group) {
		for(var i = 0;i<group.length;i++) {
			var stone = group[i];
			var stoneLiberties = findStoneLiberties(game,stone);

			if (stoneLiberties.length > 0) {
				return false; // group is alive (for now at least...)
			}
		}
		return true; // group as no more liberties... so it's dead
	}

	

	


	/* -----------------------------------------------------------
	 * Internal usage
	 */
	function addStoneToTheRightGroup (game,groups,stone) {
		
		var groupsIncontactWithStone = [];
		for(var i = 0; i < groups.length; i++) {
			var group = groups[i];
			if (stoneBelongsToGroup(group,stone)) {
				groupsIncontactWithStone.push(group);
			}
		}

		// fusion all groups to one (as stone can link several groups)
		if (groupsIncontactWithStone.length == 1) {
			groupsIncontactWithStone[0].push(stone);

		} else if (groupsIncontactWithStone.length > 1) {
			
			var fusionedGroup = [];
			for (var j= 0;j<groupsIncontactWithStone.length;j++ ) {
				var _group = groupsIncontactWithStone[j];
				for (var k= 0;k<_group.length;k++ ) {
					var s = _group[k];
					fusionedGroup.push(s);
				}
				// remove the group that just had been fusionned
				removeGroupFromGroups(groups,_group );
			}
			// add the stone we just played
			fusionedGroup.push(stone);
			// and add the fusionned existing groups
			groups.push(fusionedGroup);
		} else {
			// if not in any group, create a new one
			groups.push([stone]);
		}
	}

	function removeGroupFromGroups(groups,group) {
		
		var strRep1 = posToString(group);
		for(var i = 0;i< groups.length;i++) {
			var strRep2 = posToString(groups[i]);
			if (strRep1 == strRep2) {
				groups.splice(i, 1);
				return ;
			}
		}
	}
	function stoneBelongsToGroup(group,stone) {
		for (var i = 0; i< group.length;i++) {
			var groupStone = group[i];
			
			if (areTwoStonesConnected(stone,groupStone)) {
				return true;
			}
			
		}
		return false;
	}

	this.areTwoStonesConnected = function(stone1,stone2) {
		if (stone1.c == stone2.c) {
			if (stone1.y == stone2.y) {
				if (stone1.x==(stone2.x-1)) {return true;}
				if (stone1.x==(stone2.x+1)) {return true;}
			}
			if (stone1.x == stone2.x) {
				if (stone1.y==(stone2.y-1)) {return true;}
				if (stone1.y==(stone2.y+1)) {return true;}
			}
		}
		return false;
	}


	// up free
	function uf(game,stone) {
		if (stone.c=== 0) {
			return true; // dont care of this one
		} else {
			if (stone.y===0) {
				return false; // top stones have no up liberty
			} else {

				var upperStoneColor = game[stone.y-1][stone.x];
				if (upperStoneColor === 0 ) {
					return true;
				} else {
					return false;
				}
			}
		}
	}

	// bottom free
	function bf(game,stone) {
		if (stone.c=== 0) {
			return true; // dont care of this one
		} else {
			
			if (stone.y==(game.length-1)) {
				return false; // top stones have no bottom liberty
			} else {
				var bottomStoneColor = game[stone.y+1][stone.x];
				if (bottomStoneColor === 0 ) {
					return true;
				} else {
					return false;
				}
			}
		}
	}
	// right free
	function rf(game,stone) {
		if (stone.c=== 0) {
			return true; // dont care of this one
		} else {
			if (stone.x==(game[0].length-1)) {
				return false; // top stones have no up liberty
			} else {
				var rightStoneColor = game[stone.y][stone.x+1];
				if (rightStoneColor === 0 ) {
					return true;
				} else {
					return false;
				}
			}
		}
	}
	// left free
	function lf(game,stone) {
		if (stone.c === 0) {
			return true; // dont care of this one
		} else {
			if (stone.x===0) {
				return false; // left stones have no left liberty
			} else {
				var leftStoneColor = game[stone.y][stone.x-1];
				if (leftStoneColor === 0 ) {
					return true;
				} else {
					return false;
				}
			}
		}
	}
	return {
		buildGroups : this.buildGroups,
		areTwoStonesConnected : this.areTwoStonesConnected
	};
};