/*jshint devel:true*/
/*global require:true*/
//
// THIS PROGRAM USE THE SMARTGAME PLUGIN by neagle
// https://www.npmjs.org/package/smartgame

var args = process.argv.slice(2);
var engine = require("./engine.js")();
var fs = require("fs");
var sgf = require('smartgame');

// TODO : put that somewhere else...
String.prototype.lpad = function(padString, length) {
    var str = this;
    while (str.length < length)
        str = padString + str;
    return str;
};

/* ---------------------------------------------------------------
 * Initialize plugins
 */
var plugins = [];
var files = fs.readdirSync('./plugins/');
for (var i in files) {
	if (files[i][0] === '_') {
		// Ignore plugin that starts with _ so that we can 
		// disable a plugin easily just by changing its name...
		console.log('Plugin ignored: ' + files[i]);
	} else {
		console.log('Plugin : ' + files[i]);
		var plugin = require('./plugins/' + files[i]);
		plugins.push(plugin);
	}
}

/* ---------------------------------------------------------------
 * MAIN Script
 */
if (fs.lstatSync(args[0]).isDirectory()) {
	var files =  fs.readdirSync(args[0]);
	for (var i in files) {
		execFile(args[0]+"/"+files[i]);
	}
} else {
	execFile(args[0]);
}

// Once files/games/ have been parsed, invoke each plugin so that it can display its result. 
for(var i = 0; i< plugins.length;i++) {
	console.log("----------------------------------------------------------------------");
	console.log ("PLUGIN : "+plugins[i].getTitle().toUpperCase()+"\n");
	plugins[i].onFinish();
}
/*
 * END
 * ---------------------------------------------------------------
 */
 
 
function execFile(filename) {
	var sgfContent = fs.readFileSync(filename);
	var collection = sgf.parse(""+sgfContent);
	console.log("Found "+collection.gameTrees.length+" games in file '"+filename+"'");

	// for each game in the file :
	for (var i = 0;i< collection.gameTrees.length;i++) {
		var moves = collection.gameTrees[i];
		// TODO support rectangular board...
		// or at leat echo a message if the game is not rectangular....
		var game = engine.newGameXY(moves.nodes[0].SZ,moves.nodes[0].SZ);
		game = traverseGame(moves,game);
	}
}

/*
 * traverse a game
 */
function traverseGame(moves,game) {
	var header = moves.nodes[0];
	/*
	 * Header
	 */
	//console.log("Header : " + JSON.stringify(header,null,2));
	
	// Hanicap stones
	if (header.AB) {
		for (var i = 0; i< header.AB.length;i++) {
			var m = header.AB[i];
			var x = m.charCodeAt(0) - 97;
			var y =  (m.charCodeAt(1) - 97);
			game =  engine.update(game,{x:x,y:y,c:1}).game;
		}
	}
	if (header.AW) {
		for (var i = 0; i< header.AB.length;i++) {
			var m = header.AW[i];
			var x = m.charCodeAt(0) - 97;
			var y =  (m.charCodeAt(1) - 97);
			game =  engine.update(game,{x:x,y:y,c:2}).game;
		}
	}

	// for each move in game
	for (var i = 1;i< moves.nodes.length;i++) {
		game = onMove(moves.nodes[i],game);
	}
	// sequence
	game = traverseSeqs(moves.sequences,game);
	return game;
}

function traverseSeqs(sequences,game) {
	if (sequences) {
		var i  = 0; // only traverse the first variation
		// Move Nodes
		for (var n = 0;n< sequences[i].nodes.length;n++) {
			game = onMove(sequences[i].nodes[n], game);
		}
		// Sequences
		game = traverseSeqs(sequences[i].sequences, game);
	}
	return game;
}

function onMove(move,game) {
	var stone = asStone(move);
	var groups = [];
	var st_group = null;
	var st_groups = null;
	if (stone) {
		var gameAndGroups = engine.update(game,stone);
		game = gameAndGroups.game;
		st_groups = gameAndGroups.st_groups;

		// find stone's own group
		for (var i=0;i< st_groups.length;i++) {
			for (var j=0;j< st_groups[i].stones.length;j++) {
				var st = st_groups[i].stones[j];
				if ((st.c === stone.c) &&(st.x === stone.x)&&(st.y === stone.y)) {
					st_group = st_groups[i]; 
					break;
				}
			}
		}
	}
	/*
	 * Invoke each plugin "onMove" callback function passing the current move
	 */
	for(var k = 0; k< plugins.length;k++) {
		plugins[k].onMove(move,stone,st_group,game,st_groups);
	}

	return game;
}



