var MOVE_CHAR = "1";

exports.isPattern = function(pattern, stone, game) {
	var patterns = buildAllPatterns(pattern);
	// check each pattern variation (rotation, mirrored)
	for (var i = 0;i< patterns.length;i++) {
		if (isPatternInGame(patterns[i], stone, game)) {
			return true;
		}
	}
	return false;
};

exports.getMatchingPositions = function(pattern, stone, game) {
	
	var matching = [];
	var patterns = buildAllPatterns(pattern);
	// check each pattern variation (rotation, mirrored)
	for (var i = 0;i< patterns.length;i++) {

		if (isPatternInGame(patterns[i], stone, game)) {
			//console.log("---------------START PATTERN-------------------");
			//console.log(">>>Pattern-"+JSON.stringify(patterns[i]));

			var patternAndPos = extractGamePattern(game, stone, patterns[i]);
			// y loop on a pattern
			var positions = [];
			for (var j = 0; j < patterns[i].length; j++ ) {
				var line = patterns[i][j];
				// x loop on a pattern
				for (var k = 0;k<line.length;k++) {
					/*
					 * TODO... let's rewrite that later in a more general form ;)
					 */
					if (line[k] == '1') {
						
						positions[0] = {
							x:patternAndPos.x+k,
							y:patternAndPos.y+j
						};
						
					} else if (line[k] == '2') {
						
						positions[1] = {
							x:patternAndPos.x+k,
							y:patternAndPos.y+j
						};
						
					} else if (line[k] == '3') {
						
						positions[2] = {
							x:patternAndPos.x+k,
							y:patternAndPos.y+j
						};
						
					}
				}
				
			}
			//console.log(">>>POSITIONS-"+JSON.stringify(positions));
			matching.push(positions);
			//console.log("---------------END PATTERN----------------------");
		}
	}
	
	return matching;
};

function isPatternInGame(pattern, stone, game) {
	var region = extractGamePattern(game, stone, pattern).pattern;
	return match(stone, pattern,region);
}

function extractGamePattern(game, pos, pattern) {
	var patternWidth = pattern[0].length;
	var patternHeight = pattern.length;
	var patternCenterX = 0;
	var patternCenterY = 0;
	// find pattern "center"
	for (var j=0;j<patternHeight;j++ ) {
		for (var i=0; i< patternWidth;i++) {
			if (pattern[j][i] == MOVE_CHAR) {
				patternCenterX = i;
				patternCenterY = j;
			}
		}
	}
	return {
		x: pos.x-patternCenterX,
		y : pos.y-patternCenterY,
		pattern : extractPatternForRect(game,
			pos.x-patternCenterX,
			pos.y-patternCenterY,
			patternWidth, patternHeight)
	};
}

function match (stone, pattern, region) {

	if (region === null) {
		return false; // out of board -> don't match
	}
	
	for(var y=0;y<pattern.length;y++) {
		var oneColor = "";
		if (stone.c == 2) {
			oneColor = 'O';
		} else {
			oneColor = 'X';
		}
		for(var x=0;x<pattern[0].length;x++) {
			
			var ch1 = region[y][x];
			var ch2 = pattern[y][x];

			// 'o' specifiy that the color must be a different color then '1', or empty
			if (ch2 == 'o') {
				if (ch1 == oneColor){
					return false;
				}
			}
			// 'x' specifiy that the color must be of same color then '1', or empty
			else if (ch2 == 'x') {
				if (!((ch1 == oneColor) ||(ch1 == '.'))){
					return false;
				}
			} else  if (ch2 != '?') { // ignore '?' position... any thing match...
				// adapt pattern to move color.
				if (stone.c == 2) {
					if (ch2=="X") {
						ch2 = "O";
					} else if (ch2=="O") {
						ch2 = "X";
					} else if (ch2=="1") {
						ch2 = "O";
					} else if (ch2=="2") {
						ch2 = "X";
					} else if (ch2=="3") {
						ch2 = "O";
					}

				} else if (stone.c == 1) {
					if (ch2=="1") {
						ch2 = "X";
					}else if (ch2=="2") {
						ch2 = "O";
					} else if (ch2=="3") {
						ch2 = "X";
					}
				}

				if (ch1 != ch2) {
					return false; // pattern don't match
				}
			}
		}
	}
	return true; // pattern match
}

function extractPatternForRect(game, x, y, width, height) {
		if (x < 0 ) return null;
		if (x+width > game[0].length ) return null;
		if (y < 0 ) return null;
		if (y+height > game.length ) return null;
		var pattern = [];
		for(var j = 0; j<height; j++) {
			var line = [];
			for(var i = 0;i< width ; i++) {
				var c = game[j+y][i+x];
				var ch = ".";
				if (c ==1) {
					ch = "X";
				} else if (c ==2) {
					ch = "O";
				}
				line+=ch;
			}
			pattern.push(line);
		}
		//console.log("extractPatternForRect>"+JSON.stringify(pattern,null,2));
		return pattern;
	}
/*
 * Pattern stuff (mirror, rotations...)
 */
function buildAllPatterns(pattern) {
	// NICE TO HAVE : just keep distinct patterns to speedup things a bit...
	return [
		pattern,
		horizontalMirror(pattern),
		verticalMirror(pattern),
		horizontalMirror(verticalMirror(pattern)),
		rotation(pattern),
		rotation(horizontalMirror(pattern)),
		rotation(verticalMirror(pattern)),
		rotation(horizontalMirror(verticalMirror(pattern)))
	];
}

function horizontalMirror(pattern) {
	var res = [];
	for (var i = 0;i< pattern.length;i++) {
		var s = pattern[i];
		res.push( s.split("").reverse().join(""));
	}
	return res;
}

function verticalMirror(pattern) {
	var res = [];
	for (var i = pattern.length-1 ;i>= 0;i--) {
		var s = pattern[i];
		res.push (s);
	}
	return res;
}

function rotation(pattern) {
	var res =[];
	var first = pattern[0];
	for (var j = first.length-1 ;j>= 0; j--) {
		var str ="";
		for (var i = 0;i< pattern.length;i++) {
			str += pattern[i][j];
		}
		res.push (str);
	}
	return res;
}

/*
 * Helpfull for debug...
 */
function toString(pattern) {
	if (pattern === null) {return "pattern is null";}
	var res ="";
	for (var i = 0;i< pattern.length;i++) {
		res += pattern[i]+"\n";
	}
	return res.trim();
}
