var ascii = require("../ascii.js");
var p = require("../pattern.js");

var TITLE = "Move Statistics";
var unknowBlackMoves = [];
var unknowWhiteMoves = [];
var moveNumber = 0;
var buffer ="";
var nbGroupsInAtariPrevMove = 0;
var nbGroupsInAtari = 0;

function isKomoku (move, stone, st_group, game, st_groups) {
	// A bit subjectiv way to identify Komoku... but should be close enougth...
	
	var is34Point = (
		// top left
		( stone.x == 2  && stone.y == 3) ||
		( stone.x == 3 && stone.y == 2) ||
		// top right
		( stone.x == 2  && stone.y == 15) ||
		( stone.x == 16 && stone.y == 3) ||
		// bottom left
		( stone.x == 2  && stone.y == 15) ||
		( stone.x == 3 && stone.y == 16) ||
		// bottom right
		( stone.x == 16  && stone.y == 15) ||
		( stone.x == 15 && stone.y == 16)
	);
	return (moveNumber < 20) && is34Point;
}

function isSansan(move, stone, st_group, game, st_groups) {
	
	// 2 as we start counting à 0...
	var is33Point = (
		( stone.x == 2  && stone.y == 2) ||
		( stone.x == 16 && stone.y == 2) ||
		(stone.x == 2   && stone.y == 16) ||
		(stone.x == 16  && stone.y == 16)
	);
	return (moveNumber < 20) && is33Point;
}
function isTengen (move, stone, st_group, game, st_groups) {
	// A bit subjectiv way to identify Tengen... but should be close enougth...
	var nothingAroundPattern = [
		".....",
		".....",
		"..1..",
		".....",
		"....."
	];
	var center = (game.length-1)/2;
	var isPattern =  p.isPattern(nothingAroundPattern, stone, game) ;
	var isTengenPoint = stone.x == center && stone.y == center ;
	return isPattern && isTengenPoint;
}

function isHoshi (move, stone, st_group, game, st_groups) {
	
	// 3 as we start counting à 0...
	var is44Point = (
		(stone.x == 3  && stone.y == 3)  ||
		(stone.x == 15 && stone.y == 3)  ||
		(stone.x == 3  && stone.y == 15) ||
		(stone.x == 15 && stone.y == 15)
	);

	return is44Point && moveNumber < 20;
}

function isKakari(move, stone, st_group, game, st_groups) {
	// approch move
	var pattern = [
				".2.",
				"...",
				"1.."
				];
			
	var matching = p.getMatchingPositions(pattern, stone, game );
	// matching : [[{x,y},{x,y},...],[...],...]
	for (var i = 0;i<matching.length;i++) {
		var m = matching[i];
		var s2 = m[1];
		s2.c = 1;
		return isCornerMove(null, s2, game, st_groups);
	}
	return false;
}

function isCornerMove(move, stone, game, st_groups) {
	return (
		isHoshi(move, stone, null, game, st_groups) ||
		isKomoku(move, stone, null, game, st_groups) ||
		isSansan(move, stone, null, game, st_groups)
		// TODO : 5-3, 5-4
	);
}

var patterns = [
	{name : "Hoshi", black : 0, white : 0, is : isHoshi},
	{name : "Sansan", black : 0, white : 0,is : isSansan},
	{name : "Tengen", black : 0, white : 0,is : isTengen},
	{name : "Komoku", black : 0, white : 0,is : isKomoku},
	{name : "Kakari", black : 0, white : 0,is : isKakari},
	
	{
		name : "Nobi", black : 0, white : 0,
		is : function(move, stone, st_group, game, st_groups) {
			var pattern = [
			"oo?",
			"X1o",
			"oo?"];

			var patternFirstLine = [
			"X1",
			"oo"];
			
			var matching = p.getMatchingPositions(patternFirstLine, stone, game );
			for (var i = 0;i<matching.length;i++) {
				var m = matching[i];
				var s1 = m[0];
				var isFirstLine = (
					(s1.x === 0) ||
					(s1.y === 0) ||
					(s1.x === (game[0].length-1)) ||
					(s1.y === (game.length-1)));
				
					if( isFirstLine) {
						return true;
					}

			}

			return p.isPattern(pattern, stone, game) ;
		}
	},{
		name : "Kosumi", black : 0, white : 0,
		is : function(move, stone, st_group, game, st_groups) {
			var pattern = [
				".1",
				"X."];
			return p.isPattern(pattern, stone, game);
		}
	},{
		name : "Hane", black : 0, white : 0,
		is : function(move, stone, st_group, game, st_groups) {
			var pattern = [
				".X?",
				"1Oo"];
			return p.isPattern(pattern, stone,game);
		}
	},{
		name : "Ikken tobi", black : 0, white : 0,
		is : function(move, stone, st_group, game, st_groups) {
			var pattern = ["X.1"];
			return p.isPattern(pattern, stone,game);
		}
	},{
		name : "Niken tobi", black : 0, white : 0,
		is : function(move, stone, st_group, game, st_groups) {
			var pattern = ["X..1"];
			return p.isPattern(pattern, stone,game);
		}
	},{
		name : "Kogeima", black : 0, white : 0,
		is : function(move, stone, st_group, game, st_groups) {
			// ! means empty or 'O' (ie other color then '1')
			var p1 = [
				"..X",
				"1.."];
			var p2 = [
				"o.X",
				"1.."];
			var p3 = [
				"..X",
				"1.."];
			var p4 = [
				"..X",
				"1.o"];

			return (
					p.isPattern(p1,stone,game) ||
					p.isPattern(p2,stone,game) ||
					p.isPattern(p3,stone,game) ||
					p.isPattern(p4,stone,game)
			);
		}
	},{
		name : "Ogeima", black : 0, white : 0,
		is : function(move, stone, st_group, game, st_groups) {
			var p1 = [
				"...X",
				"1..."];
		
			return (
					p.isPattern(p1,stone,game));
		}
	},{
		name : "Daidaigeima", black : 0, white : 0,
		is : function(move, stone, st_group, game, st_groups) {
			// may be incomplet...
			var pattern = [
				"o...X",
				"1...."];
			return p.isPattern(pattern,stone,game);
		}
	},{
		name : "Cross cut", black : 0, white : 0,
		is : function(move, stone, st_group, game, st_groups) {
			var pattern = [
				"....",
				".OX.",
				".1O.",
				"...."];
			return p.isPattern(pattern, stone, game );
		}
	},{
		// Nozoki 
		name : "Peep", black : 0, white : 0,
		is : function(move, stone, st_group, game, st_groups) {
			var p1 = [
				"?.?",
				"O.O",
				"?1?"];

			// not a peep if a bamboo join (?)
			var p2 = [
				"O.O",
				"O.O",
				"?1?"];
			return p.isPattern(p1, stone, game ) && (! p.isPattern(p2, stone, game ));
		}
	},{

		name : "Cap", black : 0, white : 0,
		is : function(move, stone, st_group, game, st_groups) {
			var pattern = [
				"...",
				"...",
				".1.",
				"...",
				".2.",
				"...",
				"..."];
			
			//  1 must be from above...
			var matching = p.getMatchingPositions(pattern, stone, game );
			// matching : [[{x,y},{x,y},...],[...],...]
			for (var i = 0;i<matching.length;i++) {
				var m = matching[i];
				var s1 = m[0];
				var s2 = m[1];
				var center_y = (game.length-1)/2;
				var center_x = (game[0].length-1)/2;

				var dx1 = Math.abs(center_x - s1.x);
				var dx2 = Math.abs(center_x - s2.x);
				if (dx1 < dx2) {
					return true;
				}
				var dy1 = Math.abs(center_y - s1.y);
				var dy2 = Math.abs(center_y - s2.y);
				if (dy1 < dy2) {
					return true;
				}
					
			}
			return false;
		}
	},{
		name : "Shoulder hit", black : 0, white : 0,
		is : function(move, stone, st_group, game, st_groups) {
			var pattern = [
				"....",
				"..2.",
				".1..",
				"...."];
			
			//  1 must be from above...
			var matching = p.getMatchingPositions(pattern, stone, game );
			// matching : [[{x,y},{x,y},...],[...],...]
			for (var i = 0;i<matching.length;i++) {
				var m = matching[i];
				var s1 = m[0];
				var s2 = m[1];
				var center_y = (game.length-1)/2;
				var center_x = (game[0].length-1)/2;

				var dx1 = Math.abs(center_x - s1.x);
				var dx2 = Math.abs(center_x - s2.x);
				if (dx1 < dx2) {
					return true;
				}
				var dy1 = Math.abs(center_y - s1.y);
				var dy2 = Math.abs(center_y - s2.y);
				if (dy1 < dy2) {
					return true;
				}
			}
			return false;
		}
	},{
		name : "Tsuke", black : 0, white : 0,
		is : function(move, stone, st_group, game, st_groups) {
			var pattern = [
				"?.?",
				".O.",
				".1.",
				"x.x"];
				// TODO : don't work if 'O' is on the first line...
			return p.isPattern(pattern, stone, game );
		}
	},{
		name : "Bump", black : 0, white : 0,
		is : function(move, stone, st_group, game, st_groups) {
			var p1 = [
				"...",
				".O.",
				".1.",
				".X."];
			var p2 = [
				"...",
				".O.",
				".X.",
				".1."];

			return p.isPattern(p1, stone, game ) || p.isPattern(p2, stone, game );
		}
	},{
		name : "Cut", black : 0, white : 0,
		is : function(move, stone, st_group, game, st_groups) {
			var pattern = [
				"OX",
				"1O"];
			return p.isPattern(pattern, stone, game );
		}
	},{
		name : "Empty triangle", black : 0, white : 0,
		is : function(move, stone, st_group, game, st_groups) {
			// by far not perfect...
			var p1 = [
				
				".Xo",
				"1Xo",
				"oo?"];
			var p2 = [
				".Xo",
				"X1o",
				"oo?"];
			return p.isPattern(p1, stone, game ) || p.isPattern(p2, stone, game );
		}
	},{
		name : "Full triangle", black : 0, white : 0,
		is : function(move, stone, st_group, game, st_groups) {
			var p1 = [
				"?oo?",
				"oOXo",
				"o1Xo",
				"?oo?"];
			var p2 = [
				"?oo?",
				"oOXo",
				"oX1o",
				"?oo?"];
			return p.isPattern(p1, stone, game ) || p.isPattern(p2, stone, game );
		}
	},{
		name : "Turn", black : 0, white : 0,
		is : function(move, stone, st_group, game, st_groups) {
			var p1 = [
				"ooX?",
				"oOXo",
				"o1Xo",
				"?oo?"];
			
			return p.isPattern(p1, stone, game );
		}
	},{
		name : "Bamboo join", black : 0, white : 0,
		is : function(move, stone, st_group, game, st_groups) {
			var pattern = [
				"...",
				"X.X",
				"1.X",
				"..."];
			return p.isPattern(pattern, stone, game );
		}
	},{
		name : "Table shape", black : 0, white : 0,
		is : function(move, stone, st_group, game, st_groups) {
			var p1 = [
				"oXX",
				"...",
				"1.X"];
			var p2 = [
				"oXX",
				"...",
				"X.1"];
			return p.isPattern(p1, stone, game ) || p.isPattern(p2, stone, game );
		}
	},{
		name : "Tiger mouth", black : 0, white : 0,
		is : function(move, stone, st_group, game, st_groups) {
			var p1 = [
				"...",
				"X.X",
				".1O",
				"..o"];
			var p2 = [
				"...",
				"X.1",
				".XO",
				"..o"];
			var p3 = [
				"...",
				"1.X",
				".XO",
				"..o"];

			
			return p.isPattern(p1, stone, game ) || p.isPattern(p2, stone, game ) || p.isPattern(p3, stone, game );
		}
	},{
		name : "Ponnuki", black : 0, white : 0,
		is : function(move, stone, st_group, game, st_groups) {
			var p1 = [
				"?X?",
				"X.X",
				"?1?"];
			
			return p.isPattern(p1, stone, game ) ;
		}
	},{
		name : "Tortoise Shell", black : 0, white : 0,
		is : function(move, stone, st_group, game, st_groups) {
			var p1 = [
				"?X?",
				"X.X",
				"X.X",
				"?1?"];
			var p2 = [
				"?X?",
				"X.X",
				"X.1",
				"?X?"];
			
			return p.isPattern(p1, stone, game ) || p.isPattern(p2, stone, game ) ;
		}
	},{
		name : "Elephant eye", black : 0, white : 0,
		is : function(move, stone, st_group, game, st_groups) {
			var pattern = [
				"X..",
				"...",
				"..1"];
			return p.isPattern(pattern, stone, game );
		}
	},{
		name : "Crawl", black : 0, white : 0,
		is : function(move, stone, st_group, game, st_groups) {
			var pattern = [	"?o?",
							".13",
							"?o?"];
			
			var matching = p.getMatchingPositions(pattern, stone, game );
			for (var i = 0;i<matching.length;i++) {
				var m = matching[i];
				var s1 = m[0];
				var s3 = m[2];
				
				// both stones (1 and 3 in the pattern) are on second line.
				if ((Math.abs((game[0].length-1) - s1.x) == 1 ) &&
					(Math.abs((game[0].length-1) - s3.x) == 1 )) {
					return true;
				}
				if ((Math.abs((game.length-1) - s1.y) == 1 ) &&
					(Math.abs((game.length-1) - s3.y) == 1 )) {
					return true;
				}
			}
			
		}
	},{
		name : "Atari*", black : 0, white : 0,
		is : function(move, stone, st_group, game, st_groups) {
			// TODO : don't work if a move both create an atari a eat a stone in the same time...
			// MOVE : 176
			return (nbGroupsInAtari > nbGroupsInAtariPrevMove);

		}
	},{
		name : "Connection", black : 0, white : 0,
		is : function(move, stone, st_group, game, st_groups) {
			var p1 = [
				"X1X"
			];
			
			return p.isPattern(p1, stone, game );
		}
	}
	 

];


/*
 * End of move definition
 */

exports.getTitle = function() {
	return TITLE;
};
	
exports.onMove = function(move, stone, st_group, game, st_groups) {
	moveNumber ++;
	
	if (stone === null) {
		// "pass" move lead to null stone. just ignore
		return;
	}

	nbGroupsInAtari = countAtariGroups(st_groups);
	

	var nbPatternThatMatch = 0;
	var matchingNames = [];
	for (var i = 0;i<patterns.length;i++) {
		var pattern = patterns[i];
		if( pattern.is(move, stone, st_group, game, st_groups)) {
			matchingNames.push(pattern.name);
			nbPatternThatMatch ++;
			if (stone.c == 1) {
				pattern.black ++;
			} else {
				pattern.white ++;
			}
		}
	}
	if (nbPatternThatMatch === 0) {
		if (stone.c == 1) {
			unknowBlackMoves.push({move_number:moveNumber, move:move,stone:stone, ascii:ascii.toString(game,stone)});
		} else {
			unknowWhiteMoves.push({move_number:moveNumber, move:move,stone:stone, ascii:ascii.toString(game,stone)});
		}
	}
	buffer += "MOVE : "+moveNumber+", "+JSON.stringify(matchingNames)+"\n";
	buffer += ascii.toString(game,stone)+"\n";
	buffer += "----------------------------------------------------------\n";


	nbGroupsInAtariPrevMove = nbGroupsInAtari;
	
};

exports.onFinish = function() {
	console.log("WARNING : ");
	console.log("More a toy then a serious analysis tool... ");
	console.log("This is just a prototype, the result is still very... approximative... ");
	console.log(" ");
	
	console.log("                  black   white");
	console.log("                  -----   -----");
	
	for (var i = 0;i<patterns.length;i++) {
		var p1 = patterns[i];
		console.log(p1.name.lpad(' ',15)+" : "+p1.black.toString().lpad(' ',5)+"   "+p1.white.toString().lpad(' ',5));
	}

	
	console.log("");
	console.log("=============================================================");
	console.log("Not identified :");
	console.log("Black : "+unknowBlackMoves.length);
	/*
	for(var j = 0;j<unknowBlackMoves.length;j++) {
		var m = unknowBlackMoves[j];
		console.log("MOVE : "+m.move_number);
		console.log(m.ascii);
		
	}
	console.log("");
	*/
	
	console.log("White : "+unknowWhiteMoves.length);
	/*
	for(var j = 0;j<unknowWhiteMoves.length;j++) {
		var m = unknowWhiteMoves[j];
		console.log("MOVE : "+m.move_number);
		console.log(m.ascii);
	}
	*/

	
	// log each move
	/*
	console.log("");
	console.log("=============================================================");

	console.log("");
	console.log(buffer);
	*/

};

function countAtariGroups(st_groups) {
	var cpt = 0;
	for(var i = 0;i<st_groups.length;i++) {
		var st_group = st_groups[i];
		if (st_group.liberties.length == 1) {
			cpt ++;
		}
	}
	return cpt;
}
